import type { EventRegistry } from './EventRegistry';
import type { AnyEventHandler, SingleEventHandler, CommonEventDefinition, CommonEventDefinitionSchemaType, EventTypeNames } from './eventTypes';
export declare class DomainEventEmitter<SupportedEvents extends CommonEventDefinition[]> {
    private readonly eventRegistry;
    private readonly eventHandlerMap;
    private readonly anyHandlers;
    constructor({ eventRegistry }: {
        eventRegistry: EventRegistry<SupportedEvents>;
    });
    emit<SupportedEvent extends SupportedEvents[number]>(supportedEvent: SupportedEvent, data: Omit<CommonEventDefinitionSchemaType<SupportedEvent>, 'type'>): Promise<void>;
    /**
     * Register handler for a specific event
     */
    on<EventTypeName extends EventTypeNames<SupportedEvents[number]>>(eventTypeName: EventTypeName, handler: SingleEventHandler<SupportedEvents, EventTypeName>): void;
    /**
     * Register handler for multiple events
     */
    onMany<EventTypeName extends EventTypeNames<SupportedEvents[number]>>(eventTypeNames: EventTypeName[], handler: SingleEventHandler<SupportedEvents, EventTypeName>): void;
    /**
     * Register handler for all events supported by the emitter
     */
    onAny(handler: AnyEventHandler<SupportedEvents>): void;
    private addOnHandler;
}
