"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainEventEmitter = void 0;
const node_core_1 = require("@lokalise/node-core");
class DomainEventEmitter {
    eventRegistry;
    eventHandlerMap = {};
    anyHandlers = [];
    constructor({ eventRegistry }) {
        this.eventRegistry = eventRegistry;
    }
    async emit(supportedEvent, data) {
        const eventTypeName = supportedEvent.schema.shape.type.value;
        if (!this.eventRegistry.isSupportedEvent(eventTypeName)) {
            throw new node_core_1.InternalError({
                errorCode: 'UNKNOWN_EVENT',
                message: `Unknown event ${eventTypeName}`,
            });
        }
        const eventHandlers = this.eventHandlerMap[eventTypeName];
        // No relevant handlers are registered, we can stop processing
        if (!eventHandlers && this.anyHandlers.length === 0) {
            return;
        }
        const validatedEvent = this.eventRegistry
            .getEventDefinitionByTypeName(eventTypeName)
            .schema.parse({
            type: eventTypeName,
            ...data,
        });
        if (eventHandlers) {
            for (const handler of eventHandlers) {
                await handler.handleEvent(validatedEvent);
            }
        }
        for (const handler of this.anyHandlers) {
            await handler.handleEvent(validatedEvent);
        }
    }
    /**
     * Register handler for a specific event
     */
    on(eventTypeName, handler) {
        this.addOnHandler(eventTypeName, handler);
    }
    /**
     * Register handler for multiple events
     */
    onMany(eventTypeNames, handler) {
        for (const eventTypeName of eventTypeNames) {
            this.on(eventTypeName, handler);
        }
    }
    /**
     * Register handler for all events supported by the emitter
     */
    onAny(handler) {
        this.anyHandlers.push(handler);
    }
    addOnHandler(eventTypeName, handler) {
        if (!this.eventHandlerMap[eventTypeName]) {
            this.eventHandlerMap[eventTypeName] = [];
        }
        this.eventHandlerMap[eventTypeName].push(handler);
    }
}
exports.DomainEventEmitter = DomainEventEmitter;
//# sourceMappingURL=DomainEventEmitter.js.map