"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BASE_MESSAGE_SCHEMA = exports.MESSAGE_METADATA_SCHEMA = void 0;
const tslib_1 = require("tslib");
const zod_1 = tslib_1.__importDefault(require("zod"));
const baseEventSchemas_1 = require("../events/baseEventSchemas");
// External message metadata that describe the context in which the message was created, primarily used for debugging purposes
exports.MESSAGE_METADATA_SCHEMA = zod_1.default
    .object({
    schemaVersion: zod_1.default.string().min(1).describe('message schema version'),
    // this is always set to a service that created the message
    producedBy: zod_1.default.string().min(1).describe('app/service that produced the message'),
    // this is always propagated within the message chain. For the first message in the chain it is equal to "producedBy"
    originatedFrom: zod_1.default
        .string()
        .min(1)
        .describe('app/service that initiated entire workflow that led to creating this message'),
    // this is always propagated within the message chain.
    correlationId: zod_1.default.string().describe('unique identifier passed to all events in workflow chain'),
})
    .describe('external message metadata');
exports.BASE_MESSAGE_SCHEMA = baseEventSchemas_1.BASE_EVENT_SCHEMA.extend({
    // For internal domain events that did not originate within a message chain metadata field can be omitted, producer should then assume it is initiating a new chain
    metadata: exports.MESSAGE_METADATA_SCHEMA.optional(),
});
//# sourceMappingURL=baseMessageSchemas.js.map