import type { ErrorReporter, ErrorResolver, Either } from '@lokalise/node-core';
import type { ZodSchema, ZodType } from 'zod';
import type { MessageInvalidFormatError, MessageValidationError } from '../errors/Errors';
import type { Logger, MessageProcessingResult } from '../types/MessageQueueTypes';
import type { DeletionConfig, QueueDependencies, QueueOptions } from '../types/queueOptionsTypes';
import type { BarrierCallback, BarrierResult, Prehandler, PrehandlerResult, PreHandlingOutputs } from './HandlerContainer';
import type { HandlerSpy, PublicHandlerSpy } from './HandlerSpy';
export type Deserializer<MessagePayloadType extends object> = (message: unknown, type: ZodType<MessagePayloadType>, errorProcessor: ErrorResolver) => Either<MessageInvalidFormatError | MessageValidationError, MessagePayloadType>;
type CommonQueueLocator = {
    queueName: string;
};
export declare abstract class AbstractQueueService<MessagePayloadSchemas extends object, MessageEnvelopeType extends object, DependenciesType extends QueueDependencies, QueueConfiguration extends object, QueueLocatorType extends object = CommonQueueLocator, OptionsType extends QueueOptions<QueueConfiguration, QueueLocatorType> = QueueOptions<QueueConfiguration, QueueLocatorType>, ExecutionContext = undefined, PrehandlerOutput = undefined> {
    protected readonly errorReporter: ErrorReporter;
    readonly logger: Logger;
    protected readonly messageTypeField: string;
    protected readonly messageIdField: string;
    protected readonly logMessages: boolean;
    protected readonly creationConfig?: QueueConfiguration;
    protected readonly locatorConfig?: QueueLocatorType;
    protected readonly deletionConfig?: DeletionConfig;
    protected readonly _handlerSpy?: HandlerSpy<MessagePayloadSchemas>;
    get handlerSpy(): PublicHandlerSpy<MessagePayloadSchemas>;
    constructor({ errorReporter, logger }: DependenciesType, options: OptionsType);
    protected abstract resolveSchema(message: MessagePayloadSchemas): Either<Error, ZodSchema<MessagePayloadSchemas>>;
    protected abstract resolveMessage(message: MessageEnvelopeType): Either<MessageInvalidFormatError | MessageValidationError, unknown>;
    /**
     * Format message for logging
     */
    protected resolveMessageLog(message: MessagePayloadSchemas, _messageType: string): unknown;
    /**
     * Log preformatted and potentially presanitized message payload
     */
    protected logMessage(messageLogEntry: unknown): void;
    protected logProcessedMessage(_message: MessagePayloadSchemas | null, processingResult: MessageProcessingResult, messageId?: string): void;
    protected handleError(err: unknown, context?: Record<string, unknown>): void;
    protected handleMessageProcessed(message: MessagePayloadSchemas | null, processingResult: MessageProcessingResult, messageId?: string): void;
    protected processPrehandlersInternal(preHandlers: Prehandler<MessagePayloadSchemas, ExecutionContext, PrehandlerOutput>[], message: MessagePayloadSchemas): Promise<PrehandlerOutput>;
    protected preHandlerBarrierInternal<BarrierOutput>(barrier: BarrierCallback<MessagePayloadSchemas, ExecutionContext, PrehandlerOutput, BarrierOutput> | undefined, message: MessagePayloadSchemas, executionContext: ExecutionContext, preHandlerOutput: PrehandlerOutput): Promise<BarrierResult<BarrierOutput>>;
    protected abstract resolveNextFunction(preHandlers: Prehandler<MessagePayloadSchemas, ExecutionContext, PrehandlerOutput>[], message: MessagePayloadSchemas, index: number, preHandlerOutput: PrehandlerOutput, resolve: (value: PrehandlerOutput | PromiseLike<PrehandlerOutput>) => void, reject: (err: Error) => void): (preHandlerResult: PrehandlerResult) => void;
    protected resolveNextPreHandlerFunctionInternal(preHandlers: Prehandler<MessagePayloadSchemas, ExecutionContext, PrehandlerOutput>[], executionContext: ExecutionContext, message: MessagePayloadSchemas, index: number, preHandlerOutput: PrehandlerOutput, resolve: (value: PrehandlerOutput | PromiseLike<PrehandlerOutput>) => void, reject: (err: Error) => void): (preHandlerResult: PrehandlerResult) => void;
    protected abstract processPrehandlers(message: MessagePayloadSchemas, messageType: string): Promise<PrehandlerOutput>;
    protected abstract preHandlerBarrier<BarrierOutput>(message: MessagePayloadSchemas, messageType: string, preHandlerOutput: PrehandlerOutput): Promise<BarrierResult<BarrierOutput>>;
    protected abstract processMessage(message: MessagePayloadSchemas, messageType: string, preHandlingOutputs: PreHandlingOutputs<PrehandlerOutput, any>): Promise<Either<'retryLater', 'success'>>;
    abstract close(): Promise<unknown>;
}
export {};
