"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveHandlerSpy = exports.HandlerSpy = exports.isHandlerSpy = void 0;
const node_crypto_1 = require("node:crypto");
const node_core_1 = require("@lokalise/node-core");
const toad_cache_1 = require("toad-cache");
const matchUtils_1 = require("../utils/matchUtils");
function isHandlerSpy(value) {
    return ((0, node_core_1.isObject)(value) &&
        (value instanceof HandlerSpy || value.name === 'HandlerSpy'));
}
exports.isHandlerSpy = isHandlerSpy;
class HandlerSpy {
    name = 'HandlerSpy';
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    messageBuffer;
    messageIdField;
    spyPromises;
    constructor(params = {}) {
        this.messageBuffer = new toad_cache_1.Fifo(params.bufferSize ?? 100);
        // @ts-ignore
        this.messageIdField = params.messageIdField ?? 'id';
        this.spyPromises = [];
    }
    messageMatchesFilter(spyResult, fields, processingResult) {
        return (
        // @ts-ignore
        (0, matchUtils_1.objectMatches)(fields, spyResult.message) &&
            (!processingResult || spyResult.processingResult === processingResult));
    }
    waitForMessageWithId(id, processingResult) {
        return this.waitForMessage(
        // @ts-ignore
        {
            [this.messageIdField]: id,
        }, processingResult);
    }
    waitForMessage(fields, processingResult) {
        const processedMessageEntry = Object.values(this.messageBuffer.items).find(
        // @ts-ignore
        (spyResult) => {
            return this.messageMatchesFilter(spyResult.value, fields, processingResult);
        });
        if (processedMessageEntry) {
            return Promise.resolve(processedMessageEntry.value);
        }
        let resolve;
        const spyPromise = new Promise((_resolve) => {
            resolve = _resolve;
        });
        this.spyPromises.push({
            promise: spyPromise,
            processingResult,
            fields,
            // @ts-ignore
            resolve,
        });
        return spyPromise;
    }
    clear() {
        this.messageBuffer.clear();
    }
    addProcessedMessage(processingResult, messageId) {
        const resolvedMessageId = processingResult.message?.[this.messageIdField] ?? messageId ?? (0, node_crypto_1.randomUUID)();
        // If we failed to parse message, let's store id at least
        const resolvedProcessingResult = processingResult.message
            ? processingResult
            : {
                ...processingResult,
                message: {
                    [this.messageIdField]: messageId,
                },
            };
        // @ts-ignore
        const cacheId = `${resolvedMessageId}-${Date.now()}-${(Math.random() + 1)
            .toString(36)
            .substring(7)}`;
        this.messageBuffer.set(cacheId, resolvedProcessingResult);
        const foundPromise = this.spyPromises.find((spyPromise) => {
            return this.messageMatchesFilter(resolvedProcessingResult, spyPromise.fields, spyPromise.processingResult);
        });
        if (foundPromise) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-call
            foundPromise.resolve(processingResult);
            const index = this.spyPromises.indexOf(foundPromise);
            if (index > -1) {
                // only splice array when item is found
                this.spyPromises.splice(index, 1); // 2nd parameter means remove one item only
            }
        }
    }
}
exports.HandlerSpy = HandlerSpy;
function resolveHandlerSpy(queueOptions) {
    if (isHandlerSpy(queueOptions.handlerSpy)) {
        return queueOptions.handlerSpy;
    }
    if (!queueOptions.handlerSpy) {
        return undefined;
    }
    if (queueOptions.handlerSpy === true) {
        return new HandlerSpy();
    }
    return new HandlerSpy(queueOptions.handlerSpy);
}
exports.resolveHandlerSpy = resolveHandlerSpy;
//# sourceMappingURL=HandlerSpy.js.map