import type { TransactionObservabilityManager } from '@lokalise/node-core';
import type { ZodSchema } from 'zod';
export interface QueueConsumer {
    start(): Promise<unknown>;
    close(): Promise<unknown>;
}
export type MessageProcessingResult = 'retryLater' | 'consumed' | 'published' | 'error' | 'invalid_message';
export interface SyncPublisher<MessagePayloadType> {
    publish(message: MessagePayloadType): void;
}
export interface AsyncPublisher<MessagePayloadType, MessageOptions> {
    publish(message: MessagePayloadType, options: MessageOptions): Promise<unknown>;
}
export { TransactionObservabilityManager };
export type LogFn = {
    <T extends object>(obj: T, msg?: string, ...args: any[]): void;
    (obj: unknown, msg?: string, ...args: any[]): void;
    (msg: string, ...args: any[]): void;
};
export type ExtraParams = {
    logger?: Logger;
};
export type Logger = {
    error: LogFn;
    info: LogFn;
    warn: LogFn;
    debug: LogFn;
    trace: LogFn;
    fatal: LogFn;
};
export type SchemaMap<SupportedMessageTypes extends string> = Record<SupportedMessageTypes, ZodSchema<any>>;
